/*
 * This file automatically produced by /opt/Math/AddOns/MathLink/DeveloperKit/Linux/CompilerAdditions/mprep from:
 *	./src/cuhre/Cuhre.tm
 * mprep Revision 10 Copyright (c) Wolfram Research, Inc. 1990-2001
 */

#define MPREP_REVISION 10


int MLAbort = 0;
int MLDone  = 0;
long MLSpecialCharacter = '\0';

#include "mathlink.h"
MLINK stdlink = 0;
MLEnvironment stdenv = 0;
MLYieldFunctionObject stdyielder = 0;
MLMessageHandlerObject stdhandler = 0;

/********************************* end header *********************************/


# line 114 "./src/cuhre/Cuhre.tm"
/*
	Cuhre.tm
		Adaptive integration using cubature rules
		by Thomas Hahn
		last modified 3 Feb 05 th
*/


#include <setjmp.h>
#include "mathlink.h"
#include "util.c"

jmp_buf abort_;

/*********************************************************************/

static void Status(MLCONST char *msg, cint n1, cint n2)
{
  MLPutFunction(stdlink, "CompoundExpression", 2);
  MLPutFunction(stdlink, "Message", 3);
  MLPutFunction(stdlink, "MessageName", 2);
  MLPutSymbol(stdlink, "Cuhre");
  MLPutString(stdlink, msg);
  MLPutInteger(stdlink, n1);
  MLPutInteger(stdlink, n2);
}

/*********************************************************************/

static void Print(MLCONST char *s)
{
  int pkt;

  MLPutFunction(stdlink, "EvaluatePacket", 1);
  MLPutFunction(stdlink, "Print", 1);
  MLPutString(stdlink, s);
  MLEndPacket(stdlink);

  do {
    pkt = MLNextPacket(stdlink);
    MLNewPacket(stdlink);
  } while( pkt != RETURNPKT );
}

/*********************************************************************/

static void DoSample(cnumber n, real *x, real *f)
{
  int pkt;
  real *mma_f;
  long mma_n;

  if( MLAbort ) goto abort;

  MLPutFunction(stdlink, "EvaluatePacket", 1);
  MLPutFunction(stdlink, "Cuba`Cuhre`sample", 1);
  MLPutRealList(stdlink, x, n*ndim_);
  MLEndPacket(stdlink);

  while( (pkt = MLNextPacket(stdlink)) && (pkt != RETURNPKT) )
    MLNewPacket(stdlink);

  if( !MLGetRealList(stdlink, &mma_f, &mma_n) ) {
    MLClearError(stdlink);
    MLNewPacket(stdlink);
abort:
    MLPutFunction(stdlink, "Abort", 0);
    longjmp(abort_, 1);
  }

  if( mma_n != n*ncomp_ ) {
    MLDisownRealList(stdlink, mma_f, mma_n);
    MLPutSymbol(stdlink, "$Failed");
    longjmp(abort_, 1);
  }

  Copy(f, mma_f, n*ncomp_);
  MLDisownRealList(stdlink, mma_f, mma_n);

  neval_ += n;
}

/*********************************************************************/

#include "common.c"

void Cuhre(cint ndim, cint ncomp,
  creal epsrel, creal epsabs, cint flags, cint mineval, cint maxeval,
  cint key)
{
  ndim_ = ndim;
  ncomp_ = ncomp;

  if( BadComponent(ncomp) ) {
    Status("badcomp", ncomp, 0);
    MLPutSymbol(stdlink, "$Failed");
  }
  else if( BadDimension(ndim) ) {
    Status("baddim", ndim, 0);
    MLPutSymbol(stdlink, "$Failed");
  }
  else {
    real integral[NCOMP], error[NCOMP], prob[NCOMP];
    count comp;
    int fail;

    neval_ = 0;

    fail = Integrate(epsrel, Max(epsabs, NOTZERO),
      flags, mineval, maxeval, key,
      integral, error, prob);

    Status(fail ? "accuracy" : "success", neval_, nregions_);

    MLPutFunction(stdlink, "List", ncomp);
    for( comp = 0; comp < ncomp; ++comp ) {
      real res[] = {integral[comp], error[comp], prob[comp]};
      MLPutRealList(stdlink, res, Elements(res));
    }
  }

  MLEndPacket(stdlink);
}

/*********************************************************************/

int main(int argc, char **argv)
{
  return MLMain(argc, argv);
}

# line 156 "./src/cuhre/Cuhre.tm.c"


void Cuhre P(( int _tp1, int _tp2, double _tp3, double _tp4, int _tp5, int _tp6, int _tp7, int _tp8));

#if MLPROTOTYPES
static int _tr0( MLINK mlp)
#else
static int _tr0(mlp) MLINK mlp;
#endif
{
	int	res = 0;
	int _tp1;
	int _tp2;
	double _tp3;
	double _tp4;
	int _tp5;
	int _tp6;
	int _tp7;
	int _tp8;
	if ( ! MLGetInteger( mlp, &_tp1) ) goto L0;
	if ( ! MLGetInteger( mlp, &_tp2) ) goto L1;
	if ( ! MLGetReal( mlp, &_tp3) ) goto L2;
	if ( ! MLGetReal( mlp, &_tp4) ) goto L3;
	if ( ! MLGetInteger( mlp, &_tp5) ) goto L4;
	if ( ! MLGetInteger( mlp, &_tp6) ) goto L5;
	if ( ! MLGetInteger( mlp, &_tp7) ) goto L6;
	if ( ! MLGetInteger( mlp, &_tp8) ) goto L7;
	if ( ! MLNewPacket(mlp) ) goto L8;

	Cuhre(_tp1, _tp2, _tp3, _tp4, _tp5, _tp6, _tp7, _tp8);

	res = 1;
L8: L7: L6: L5: L4: L3: L2: L1: 
L0:	return res;
} /* _tr0 */


static struct func {
	int   f_nargs;
	int   manual;
	int   (*f_func)P((MLINK));
	char  *f_name;
	} _tramps[1] = {
		{ 8, 0, _tr0, "Cuhre" }
		};

static char* evalstrs[] = {
	"BeginPackage[\"Cuba`\"]",
	(char*)0,
	"Cuhre::usage =  \"Cuhre[f, {x, xmin, xmax}..] computes a numerica",
	"l approximation to the integral of the real scalar or vector fun",
	"ction f.  The output is a list with entries of the form {integra",
	"l, error, chi-square probability} for each component of the inte",
	"grand.\"",
	(char*)0,
	"Key::usage = \"Key is an option of Cuhre.  It specifies the basic",
	" integration rule:\\n  7 = use a degree-7 rule,\\n  9 = use a degree",
	"-9 rule,\\n  11 = use a degree-11 rule (available only in 3 dimens",
	"ions),\\n  13 = use a degree-13 rule (available only in 2 dimensio",
	"ns),\\n  otherwise the default rule is used: the degree-13 rule in",
	" 2 dimensions, the degree-11 rule in 3 dimensions, else the degr",
	"ee-9 rule.\"",
	(char*)0,
	"MinPoints::usage = \"MinPoints is an option of Cuhre.  It specifi",
	"es the minimum number of points to sample.\"",
	(char*)0,
	"Final::usage = \"Final is an option of Cuhre.  It can take the va",
	"lues Last or All which determine whether only the last (largest)",
	" or all sets of samples collected on a subregion over the iterat",
	"ions contribute to the final result.\"",
	(char*)0,
	"Regions::usage = \"Regions is an option of Cuhre.  It specifies w",
	"hether the regions into which the integration region has been cu",
	"t are returned together with the integration results.\"",
	(char*)0,
	"Region::usage = \"Region[ll, ur, res] describes a subregion:  ll ",
	"and ur are multidimensional equivalents of the region's lower le",
	"ft and upper right corner.  res gives the integration results fo",
	"r the region in a list with entries of the form {integral, error",
	"} for each component of the integrand.\"",
	(char*)0,
	"Begin[\"`Cuhre`\"]",
	(char*)0,
	"Attributes[Cuhre] = {HoldFirst}",
	(char*)0,
	"Options[Cuhre] = {PrecisionGoal -> 3, AccuracyGoal -> 12,  MinPo",
	"ints -> 0, MaxPoints -> 50000, Key -> 0,  Verbose -> 1, Final ->",
	" Last, Regions -> False, Compiled -> True}",
	(char*)0,
	"Cuhre[f_, v:{_, _, _}.., opt___Rule] :=  Block[ {ff = HoldForm[f",
	"], ndim = Length[{v}],  tags, vars, lower, range, jac, tmp, defs",
	", integrand,  rel, abs, mineval, maxeval, key, verbose, final, r",
	"egions, compiled},    Message[Cuhre::optx, #, Cuhre]&/@      Com",
	"plement[First/@ {opt}, tags = First/@ Options[Cuhre]];    {rel, ",
	"abs, mineval, maxeval, key,      verbose, final, regions, compil",
	"ed} =      tags /. {opt} /. Options[Cuhre];    {vars, lower, ran",
	"ge} = Transpose[{v}];    jac = Simplify[Times@@ (range -= lower)",
	"];    tmp = Array[tmpvar, ndim];    defs = Simplify[lower + rang",
	"e tmp];    Block[{Set}, define[compiled, tmp, vars, Thread[vars ",
	"= defs], jac]];    integrand = fun[f];    MLCuhre[ndim, ncomp[f]",
	", 10.^-rel, 10.^-abs,      Min[Max[verbose, 0], 3] +        If[f",
	"inal === Last, 4, 0] +        If[TrueQ[regions], 256, 0],       ",
	"      mineval, maxeval, key]  ]",
	(char*)0,
	"tmpvar[n_] := ToExpression[\"Cuba`Cuhre`t\" <> ToString[n]]",
	(char*)0,
	"Attributes[ncomp] = Attributes[fun] = {HoldAll}",
	(char*)0,
	"ncomp[f_List] := Length[f]",
	(char*)0,
	"_ncomp = 1",
	(char*)0,
	"define[True, tmp_, vars_, {defs__}, jac_] :=  fun[f_] := Compile",
	"[tmp, Block[vars, defs; check[vars, Chop[f jac]//N]]]",
	(char*)0,
	"define[False, tmp_, vars_, {defs__}, jac_] :=  fun[f_] := Functi",
	"on[tmp, Block[vars, defs; check[vars, Chop[f jac]//N]]]",
	(char*)0,
	"check[_, f_Real] = {f}",
	(char*)0,
	"check[_, f:{__Real}] = f",
	(char*)0,
	"check[x_, _] := (Message[Cuhre::badsample, ff, x]; {})",
	(char*)0,
	"sample[x_] :=  Check[Apply[integrand, Partition[x, ndim], 1]//Fl",
	"atten, {}]",
	(char*)0,
	"region[ll_, ur_, r___] :=  Region[lower + range ll, lower + rang",
	"e ur, r]",
	(char*)0,
	"Cuhre::badsample = \"`` is not a real-valued function at ``.\"",
	(char*)0,
	"Cuhre::baddim = \"Cannot integrate in `` dimensions.\"",
	(char*)0,
	"Cuhre::badcomp = \"Cannot integrate `` components.\"",
	(char*)0,
	"Cuhre::accuracy =  \"Desired accuracy was not reached within `` f",
	"unction evaluations on `` subregions.\"",
	(char*)0,
	"Cuhre::success = \"Needed `` function evaluations on `` subregion",
	"s.\"",
	(char*)0,
	"End[]",
	(char*)0,
	"EndPackage[]",
	(char*)0,
	(char*)0
};
#define CARDOF_EVALSTRS 29

static int _definepattern P(( MLINK, char*, char*, int));

static int _doevalstr P(( MLINK, int));

int  _MLDoCallPacket P(( MLINK, struct func[], int));


#if MLPROTOTYPES
int MLInstall( MLINK mlp)
#else
int MLInstall(mlp) MLINK mlp;
#endif
{
	int _res;
	_res = MLConnect(mlp);
	if (_res) _res = _doevalstr( mlp, 0);
	if (_res) _res = _doevalstr( mlp, 1);
	if (_res) _res = _doevalstr( mlp, 2);
	if (_res) _res = _doevalstr( mlp, 3);
	if (_res) _res = _doevalstr( mlp, 4);
	if (_res) _res = _doevalstr( mlp, 5);
	if (_res) _res = _doevalstr( mlp, 6);
	if (_res) _res = _doevalstr( mlp, 7);
	if (_res) _res = _definepattern(mlp, "MLCuhre[ndim_, ncomp_,   epsrel_, epsabs_, flags_, mineval_, maxeval_,   key_]", "{ndim, ncomp,   epsrel, epsabs, flags, mineval, maxeval,   key}", 0);
	if (_res) _res = _doevalstr( mlp, 8);
	if (_res) _res = _doevalstr( mlp, 9);
	if (_res) _res = _doevalstr( mlp, 10);
	if (_res) _res = _doevalstr( mlp, 11);
	if (_res) _res = _doevalstr( mlp, 12);
	if (_res) _res = _doevalstr( mlp, 13);
	if (_res) _res = _doevalstr( mlp, 14);
	if (_res) _res = _doevalstr( mlp, 15);
	if (_res) _res = _doevalstr( mlp, 16);
	if (_res) _res = _doevalstr( mlp, 17);
	if (_res) _res = _doevalstr( mlp, 18);
	if (_res) _res = _doevalstr( mlp, 19);
	if (_res) _res = _doevalstr( mlp, 20);
	if (_res) _res = _doevalstr( mlp, 21);
	if (_res) _res = _doevalstr( mlp, 22);
	if (_res) _res = _doevalstr( mlp, 23);
	if (_res) _res = _doevalstr( mlp, 24);
	if (_res) _res = _doevalstr( mlp, 25);
	if (_res) _res = _doevalstr( mlp, 26);
	if (_res) _res = _doevalstr( mlp, 27);
	if (_res) _res = _doevalstr( mlp, 28);
	if (_res) _res = MLPutSymbol( mlp, "End");
	if (_res) _res = MLFlush( mlp);
	return _res;
} /* MLInstall */


#if MLPROTOTYPES
int MLDoCallPacket( MLINK mlp)
#else
int MLDoCallPacket( mlp) MLINK mlp;
#endif
{
	return _MLDoCallPacket( mlp, _tramps, 1);
} /* MLDoCallPacket */

/******************************* begin trailer ********************************/

#ifndef EVALSTRS_AS_BYTESTRINGS
#	define EVALSTRS_AS_BYTESTRINGS 1
#endif


#if CARDOF_EVALSTRS
#if MLPROTOTYPES
static int  _doevalstr( MLINK mlp, int n)
#else
static int  _doevalstr( mlp, n)
	 MLINK mlp; int n;
#endif
{
	long bytesleft, charsleft, bytesnow;
#if !EVALSTRS_AS_BYTESTRINGS
	long charsnow;
#endif
	char **s, **p;
	char *t;

	s = evalstrs;
	while( n-- > 0){
		if( *s == 0) break;
		while( *s++ != 0){}
	}
	if( *s == 0) return 0;
	bytesleft = 0;
	charsleft = 0;
	p = s;
	while( *p){
		t = *p; while( *t) ++t;
		bytesnow = t - *p;
		bytesleft += bytesnow;
		charsleft += bytesnow;
#if !EVALSTRS_AS_BYTESTRINGS
		t = *p;
		charsleft -= MLCharacterOffset( &t, t + bytesnow, bytesnow);
		/* assert( t == *p + bytesnow); */
#endif
		++p;
	}


	MLPutNext( mlp, MLTKSTR);
#if EVALSTRS_AS_BYTESTRINGS
	p = s;
	while( *p){
		t = *p; while( *t) ++t;
		bytesnow = t - *p;
		bytesleft -= bytesnow;
		MLPut8BitCharacters( mlp, bytesleft, (unsigned char*)*p, bytesnow);
		++p;
	}
#else
	MLPut7BitCount( mlp, charsleft, bytesleft);
	p = s;
	while( *p){
		t = *p; while( *t) ++t;
		bytesnow = t - *p;
		bytesleft -= bytesnow;
		t = *p;
		charsnow = bytesnow - MLCharacterOffset( &t, t + bytesnow, bytesnow);
		/* assert( t == *p + bytesnow); */
		charsleft -= charsnow;
		MLPut7BitCharacters(  mlp, charsleft, *p, bytesnow, charsnow);
		++p;
	}
#endif
	return MLError( mlp) == MLEOK;
}
#endif /* CARDOF_EVALSTRS */


#if MLPROTOTYPES
static int  _definepattern( MLINK mlp, char *patt, char *args, int func_n)
#else
static int  _definepattern( mlp, patt, args, func_n)
	MLINK  mlp;
	char  *patt, *args;
	int    func_n;
#endif
{
	MLPutFunction( mlp, "DefineExternal", (long)3);
	  MLPutString( mlp, patt);
	  MLPutString( mlp, args);
	  MLPutInteger( mlp, func_n);
	return !MLError(mlp);
} /* _definepattern */


#if MLPROTOTYPES
int _MLDoCallPacket( MLINK mlp, struct func functable[], int nfuncs)
#else
int _MLDoCallPacket( mlp, functable, nfuncs)
	MLINK mlp;
	struct func functable[];
	int nfuncs;
#endif
{
	long len;
	int n, res = 0;
	struct func* funcp;

	if( ! MLGetInteger( mlp, &n) ||  n < 0 ||  n >= nfuncs) goto L0;
	funcp = &functable[n];

	if( funcp->f_nargs >= 0
	&& ( ! MLCheckFunction(mlp, "List", &len)
	     || ( !funcp->manual && (len != funcp->f_nargs))
	     || (  funcp->manual && (len <  funcp->f_nargs))
	   )
	) goto L0;

	stdlink = mlp;
	res = (*funcp->f_func)( mlp);

L0:	if( res == 0)
		res = MLClearError( mlp) && MLPutSymbol( mlp, "$Failed");
	return res && MLEndPacket( mlp) && MLNewPacket( mlp);
} /* _MLDoCallPacket */


#if MLPROTOTYPES
mlapi_packet MLAnswer( MLINK mlp)
#else
mlapi_packet MLAnswer( mlp)
	MLINK mlp;
#endif
{
	mlapi_packet pkt = 0;

	while( !MLDone && !MLError(mlp) && (pkt = MLNextPacket(mlp), pkt) && pkt == CALLPKT){
		MLAbort = 0;
		if( !MLDoCallPacket(mlp)) pkt = 0;
	}
	MLAbort = 0;
	return pkt;
} /* MLAnswer */



/*
	Module[ { me = $ParentLink},
		$ParentLink = contents of RESUMEPKT;
		Message[ MessageName[$ParentLink, "notfe"], me];
		me]
*/

#if MLPROTOTYPES
static int refuse_to_be_a_frontend( MLINK mlp)
#else
static int refuse_to_be_a_frontend( mlp)
	MLINK mlp;
#endif
{
	int pkt;

	MLPutFunction( mlp, "EvaluatePacket", 1);
	  MLPutFunction( mlp, "Module", 2);
	    MLPutFunction( mlp, "List", 1);
		  MLPutFunction( mlp, "Set", 2);
		    MLPutSymbol( mlp, "me");
	        MLPutSymbol( mlp, "$ParentLink");
	  MLPutFunction( mlp, "CompoundExpression", 3);
	    MLPutFunction( mlp, "Set", 2);
	      MLPutSymbol( mlp, "$ParentLink");
	      MLTransferExpression( mlp, mlp);
	    MLPutFunction( mlp, "Message", 2);
	      MLPutFunction( mlp, "MessageName", 2);
	        MLPutSymbol( mlp, "$ParentLink");
	        MLPutString( mlp, "notfe");
	      MLPutSymbol( mlp, "me");
	    MLPutSymbol( mlp, "me");
	MLEndPacket( mlp);

	while( (pkt = MLNextPacket( mlp), pkt) && pkt != SUSPENDPKT)
		MLNewPacket( mlp);
	MLNewPacket( mlp);
	return MLError( mlp) == MLEOK;
}


#if MLPROTOTYPES
int MLEvaluate( MLINK mlp, charp_ct s)
#else
int MLEvaluate( mlp, s)
	MLINK mlp;
	charp_ct s;
#endif
{
	if( MLAbort) return 0;
	return MLPutFunction( mlp, "EvaluatePacket", 1L)
		&& MLPutFunction( mlp, "ToExpression", 1L)
		&& MLPutString( mlp, s)
		&& MLEndPacket( mlp);
} /* MLEvaluate */


#if MLPROTOTYPES
int MLEvaluateString( MLINK mlp, charp_ct s)
#else
int MLEvaluateString( mlp, s)
	MLINK mlp;
	charp_ct s;
#endif
{
	int pkt;
	if( MLAbort) return 0;
	if( MLEvaluate( mlp, s)){
		while( (pkt = MLAnswer( mlp), pkt) && pkt != RETURNPKT)
			MLNewPacket( mlp);
		MLNewPacket( mlp);
	}
	return MLError( mlp) == MLEOK;
} /* MLEvaluateString */


#if MLPROTOTYPES
void MLDefaultHandler( MLINK mlp, unsigned long message, unsigned long n)
#else
void MLDefaultHandler( mlp, message, n)
	MLINK mlp;
	unsigned long message, n;
#endif
{
	switch (message){
	case MLTerminateMessage:
		MLDone = 1;
	case MLInterruptMessage:
	case MLAbortMessage:
		MLAbort = 1;
	default:
		return;
	}
}

#if MLPROTOTYPES
static int _MLMain( charpp_ct argv, charpp_ct argv_end, charp_ct commandline)
#else
static int _MLMain( argv, argv_end, commandline)
  charpp_ct argv, argv_end;
  charp_ct commandline;
#endif
{
	MLINK mlp;
	long err;

	if( !stdenv)
		stdenv = MLInitialize( (MLParametersPointer)0);
	if( stdenv == (MLEnvironment)0) goto R0;

	if( !stdhandler)
		stdhandler = MLCreateMessageHandler( stdenv, MLDefaultHandler, 0);


	mlp = commandline
		? MLOpenString( stdenv, commandline, &err)
		: MLOpenArgv( stdenv, argv, argv_end, &err);
	if( mlp == (MLINK)0){
		MLAlert( stdenv, MLErrorString( stdenv, err));
		goto R1;
	}

	if( stdyielder) MLSetYieldFunction( mlp, stdyielder);
	if( stdhandler) MLSetMessageHandler( mlp, stdhandler);

	if( MLInstall( mlp))
		while( MLAnswer( mlp) == RESUMEPKT){
			if( ! refuse_to_be_a_frontend( mlp)) break;
		}

	MLClose( mlp);
R1:	MLDeinitialize( stdenv);
	stdenv = (MLEnvironment)0;
R0:	return !MLDone;
} /* _MLMain */


#if MLPROTOTYPES
int MLMainString( charp_ct commandline)
#else
int MLMainString( commandline)  charp_ct commandline;
#endif
{
	return _MLMain( (charpp_ct)0, (charpp_ct)0, commandline);
}

#if MLPROTOTYPES
int MLMainArgv( char** argv, char** argv_end) /* note not FAR pointers */
#else
int MLMainArgv( argv, argv_end)  char **argv, **argv_end;
#endif
{   
	static char FAR * far_argv[128];
	int count = 0;
	
	while(argv < argv_end)
		far_argv[count++] = *argv++;
		 
	return _MLMain( far_argv, far_argv + count, (charp_ct)0);

}

#if MLPROTOTYPES
int MLMain( int argc, charpp_ct argv)
#else
int MLMain( argc, argv) int argc; charpp_ct argv;
#endif
{
 	return _MLMain( argv, argv + argc, (charp_ct)0);
}
 
