      subroutine cuts(p3,p4,qph,icut)
!! routine where experimental cuts are applied
* Inputs:
* ======
* p3(0:3) = (E,px,py,pz) = lepton momentum
* p4(0:3) = (E,px,py,pz) = neutrino momentum (lepton momentum for Z production)
* qph(40,0:3) = momenta of the emitted photons
*
* Outputs:
* =======
* icut = 0 ---> event accepted
* icut = 1 ---> event rejected
      include 'shared.inc'
      common/fordebugging/idebugging
      common/yminimo/ymin ! for x1,x2pdf sampling
!! Adapted to ATLAS requirements

      icut = 1 ! event rejected

      zmasscut = 60.d0

      yminguess = ((zmasscut)/2.d0/ebeam)**2
c      yminguess = ((ptmin+ptminmiss)/2.d0/ebeam)**2
      ymin = max(yminguess,xpdfmin*xpdfmin)

!! no cuts
c      icut = 0
c      return
!! no cuts

** invariant mass cut
      do k = 0,3
         ptmp(k) = p3(k)+p4(k)
      enddo
      zmassinv = sqrt(dot(ptmp,ptmp))
      if (zmassinv.lt.zmasscut) return

** PT CUT *****************************************
      ptl    = sqrt(p3(1)**2+p3(2)**2)
      ptmiss = sqrt(p4(1)**2+p4(2)**2)
      if (ptl.lt.ptmin.and.ptmiss.lt.ptmin) return

** RAPIDITY CUT ****************************************
      eta1 = abs(pseudorapidity(p3))
      eta2 = abs(pseudorapidity(p4))
      if (eta1.gt.etamax.and.eta2.gt.etamax) return

      icut = 0 ! event accepted

      return
      end

*********************
*********************
*********************
*********************

! BEGIN WRITING
      subroutine cuts_for_HORACE_authors(p3,p4,qph,icut)
!! routine where experimental cuts are applied
* Inputs:
* ======
* p3(0:3) = (E,px,py,pz) = lepton momentum
* p4(0:3) = (E,px,py,pz) = neutrino momentum (lepton momentum for Z production)
* qph(40,0:3) = momenta of the emitted photons
*
* Outputs:
* =======
* icut = 0 ---> event accepted
* icut = 1 ---> event rejected
      include 'shared.inc'
      common/fordebugging/idebugging
      common/yminimo/ymin ! for x1,x2pdf sampling
      common/xpdfminmax/xamin,xamax,xbmin,xbmax
      data xamin,xamax,xbmin,xbmax /1.d0,0.d0,1.d0,0.d0/
      dimension pippo(0:3),pluto(0:3),topolino(0:3)

      icut = 1

      yminguess = ((ptmin+ptminmiss)/2.d0/ebeam)**2
      if (icutmt.eq.1) yminguess = (tmmin/2.d0/ebeam)**2

      ymin = max(yminguess,xpdfmin*xpdfmin)


!! no cuts
c      icut = 0
c      return
!! no cuts

      if ((ismear+icut).ge.1) then
         call recombine_and_smear_leshouches(p3,p4,qph,ierr)
         if (ierr.gt.0) return
      endif

** PT CUT *****************************************
      ptl    = sqrt(p3(1)**2+p3(2)**2)
      ptmiss = sqrt(p4(1)**2+p4(2)**2)
      if (ptl.lt.ptmin) then
         return
      endif
      if (ptmiss.lt.ptminmiss) then
         return
      endif
*---- MT CUT
      if (icutmt.eq.1.and.boson.eq.'W') then
         pippo(3) = 0.d0
         pluto(3) = 0.d0
         do k = 1,2
            pippo(k) = p3(k)
            pluto(k) = p4(k)
         enddo
         pippo(0) = sqrt(pippo(1)**2+pippo(2)**2)
         pluto(0) = sqrt(pluto(1)**2+pluto(2)**2)
         do k = 0,3
            topolino(k) = pippo(k) + pluto(k)
         enddo
         tm = sqrt(abs(dot(topolino,topolino)))
         if (tm.lt.tmmin) return
      endif
** RAPIDITY CUT ****************************************
      eta1 = abs(pseudorapidity(p3))
      eta2 = abs(pseudorapidity(p4))
      if (eta1.gt.etamax) then
         return
      endif
      if (boson.eq.'Z') then
         if (eta2.gt.etamax) return
         if (icutmt.eq.1) then
            do k = 0,3
               pippo(k) = p3(k)+p4(k)
            enddo
            if (sqrt(dot(pippo,pippo)).lt.tmmin) return
         endif
      endif
      icut = 0
      return
      end
! END WRITING
*********************
