*
*-----SUBROUTINE RANLUX: RANDOM NUMBER GENERATOR
*
      SUBROUTINE RANLUX(RVEC,LENV)
C         SUBTRACT-AND-BORROW RANDOM NUMBER GENERATOR PROPOSED BY
C         MARSAGLIA AND ZAMAN, IMPLEMENTED BY F. JAMES WITH THE NAME
C         RCARRY IN 1991, AND LATER IMPROVED BY MARTIN LUESCHER
C         IN 1993 TO PRODUCE "LUXURY PSEUDORANDOM NUMBERS".
C     FORTRAN 77 CODED BY F. JAMES, 1993
C
C   LUXURY LEVELS.
C   ------ ------      THE AVAILABLE LUXURY LEVELS ARE:
C
C  LEVEL 0  (P=24): EQUIVALENT TO THE ORIGINAL RCARRY OF MARSAGLIA
C           AND ZAMAN, VERY LONG PERIOD, BUT FAILS MANY TESTS.
C  LEVEL 1  (P=48): CONSIDERABLE IMPROVEMENT IN QUALITY OVER LEVEL 0,
C           NOW PASSES THE GAP TEST, BUT STILL FAILS SPECTRAL TEST.
C  LEVEL 2  (P=97): PASSES ALL KNOWN TESTS, BUT THEORETICALLY STILL
C           DEFECTIVE.
C  LEVEL 3  (P=223): DEFAULT VALUE.  ANY THEORETICALLY POSSIBLE
C           CORRELATIONS HAVE VERY SMALL CHANCE OF BEING OBSERVED.
C  LEVEL 4  (P=389): HIGHEST POSSIBLE LUXURY, ALL 24 BITS CHAOTIC.
C
C!!! ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C!!!  CALLING SEQUENCES FOR RANLUX:                                  ++
C!!!      CALL RANLUX (RVEC, LEN)   RETURNS A VECTOR RVEC OF LEN     ++
C!!!                   32-BIT RANDOM FLOATING POINT NUMBERS BETWEEN  ++
C!!!                   ZERO (NOT INCLUDED) AND ONE (ALSO NOT INCL.). ++
C!!!      CALL RLUXGO(LUX,INT,K1,K2) INITIALIZES THE GENERATOR FROM  ++
C!!!               ONE 32-BIT INTEGER INT AND SETS LUXURY LEVEL LUX  ++
C!!!               WHICH IS INTEGER BETWEEN ZERO AND MAXLEV, OR IF   ++
C!!!               LUX .GT. 24, IT SETS P=LUX DIRECTLY.  K1 AND K2   ++
C!!!               SHOULD BE SET TO ZERO UNLESS RESTARTING AT A BREAK++
C!!!               POINT GIVEN BY OUTPUT OF RLUXAT (SEE RLUXAT).     ++
C!!!      CALL RLUXAT(LUX,INT,K1,K2) GETS THE VALUES OF FOUR INTEGERS++
C!!!               WHICH CAN BE USED TO RESTART THE RANLUX GENERATOR ++
C!!!               AT THE CURRENT POINT BY CALLING RLUXGO.  K1 AND K2++
C!!!               SPECIFY HOW MANY NUMBERS WERE GENERATED SINCE THE ++
C!!!               INITIALIZATION WITH LUX AND INT.  THE RESTARTING  ++
C!!!               SKIPS OVER  K1+K2*E9   NUMBERS, SO IT CAN BE LONG.++
C!!!   A MORE EFFICIENT BUT LESS CONVENIENT WAY OF RESTARTING IS BY: ++
C!!!      CALL RLUXIN(ISVEC)    RESTARTS THE GENERATOR FROM VECTOR   ++
C!!!                   ISVEC OF 25 32-BIT INTEGERS (SEE RLUXUT)      ++
C!!!      CALL RLUXUT(ISVEC)    OUTPUTS THE CURRENT VALUES OF THE 25 ++
C!!!                 32-BIT INTEGER SEEDS, TO BE USED FOR RESTARTING ++
C!!!      ISVEC MUST BE DIMENSIONED 25 IN THE CALLING PROGRAM        ++
C!!! ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      DIMENSION RVEC(LENV)
      DIMENSION SEEDS(24), ISEEDS(24), ISDEXT(25)
carlo      PARAMETER (MAXLEV=4, LXDFLT=3)
      PARAMETER (MAXLEV=4, LXDFLT=4)
      DIMENSION NDSKIP(0:MAXLEV)
      DIMENSION NEXT(24)
      PARAMETER (TWOP12=4096., IGIGA=1000000000,JSDFLT=314159265)
      PARAMETER (ITWO24=2**24, ICONS=2147483563)
      SAVE NOTYET, I24, J24, CARRY, SEEDS, TWOM24, TWOM12, LUXLEV
      SAVE NSKIP, NDSKIP, IN24, NEXT, KOUNT, MKOUNT, INSEED
      INTEGER LUXLEV
      LOGICAL NOTYET
      DATA NOTYET, LUXLEV, IN24, KOUNT, MKOUNT /.TRUE., LXDFLT, 0,0,0/
      DATA I24,J24,CARRY/24,10,0./
C                               DEFAULT
C  LUXURY LEVEL   0     1     2   *3*    4
      DATA NDSKIP/0,   24,   73,  199,  365 /
CORRESPONDS TO P=24    48    97   223   389
C     TIME FACTOR 1     2     3     6    10   ON SLOW WORKSTATION
C                 1    1.5    2     3     5   ON FAST MAINFRAME
C
C  NOTYET IS .TRUE. IF NO INITIALIZATION HAS BEEN PERFORMED YET.
C              DEFAULT INITIALIZATION BY MULTIPLICATIVE CONGRUENTIAL
      IF (NOTYET) THEN
         NOTYET = .FALSE.
         JSEED = JSDFLT
         INSEED = JSEED
         WRITE(6,'(A,I12)') ' RANLUX DEFAULT INITIALIZATION: ',JSEED
         LUXLEV = LXDFLT
         NSKIP = NDSKIP(LUXLEV)
         LP = NSKIP + 24
         IN24 = 0
         KOUNT = 0
         MKOUNT = 0
         WRITE(6,'(A,I2,A,I4)')  ' RANLUX DEFAULT LUXURY LEVEL =  ',
     +        LUXLEV,'      P =',LP
            TWOM24 = 1.
         DO 25 I= 1, 24
            TWOM24 = TWOM24 * 0.5
         K = JSEED/53668
         JSEED = 40014*(JSEED-K*53668) -K*12211
         IF (JSEED .LT. 0)  JSEED = JSEED+ICONS
         ISEEDS(I) = MOD(JSEED,ITWO24)
   25    CONTINUE
         TWOM12 = TWOM24 * 4096.
         DO 50 I= 1,24
         SEEDS(I) = REAL(ISEEDS(I))*TWOM24
         NEXT(I) = I-1
   50    CONTINUE
         NEXT(1) = 24
         I24 = 24
         J24 = 10
         CARRY = 0.
         IF (SEEDS(24) .EQ. 0.) CARRY = TWOM24
      ENDIF
C
C          THE GENERATOR PROPER: "SUBTRACT-WITH-BORROW",
C          AS PROPOSED BY MARSAGLIA AND ZAMAN,
C          FLORIDA STATE UNIVERSITY, MARCH, 1989
C
      DO 100 IVEC= 1, LENV
      UNI = SEEDS(J24) - SEEDS(I24) - CARRY
      IF (UNI .LT. 0.)  THEN
         UNI = UNI + 1.0
         CARRY = TWOM24
      ELSE
         CARRY = 0.
      ENDIF
      SEEDS(I24) = UNI
      I24 = NEXT(I24)
      J24 = NEXT(J24)
      RVEC(IVEC) = UNI
C  SMALL NUMBERS (WITH LESS THAN 12 "SIGNIFICANT" BITS) ARE "PADDED".
      IF (UNI .LT. TWOM12)  THEN
         RVEC(IVEC) = RVEC(IVEC) + TWOM24*SEEDS(J24)
C        AND ZERO IS FORBIDDEN IN CASE SOMEONE TAKES A LOGARITHM
         IF (RVEC(IVEC) .EQ. 0.)  RVEC(IVEC) = TWOM24*TWOM24
      ENDIF
C        SKIPPING TO LUXURY.  AS PROPOSED BY MARTIN LUSCHER.
      IN24 = IN24 + 1
      IF (IN24 .EQ. 24)  THEN
         IN24 = 0
         KOUNT = KOUNT + NSKIP
         DO 90 ISK= 1, NSKIP
         UNI = SEEDS(J24) - SEEDS(I24) - CARRY
         IF (UNI .LT. 0.)  THEN
            UNI = UNI + 1.0
            CARRY = TWOM24
         ELSE
            CARRY = 0.
         ENDIF
         SEEDS(I24) = UNI
         I24 = NEXT(I24)
         J24 = NEXT(J24)
   90    CONTINUE
      ENDIF
  100 CONTINUE
      KOUNT = KOUNT + LENV
      IF (KOUNT .GE. IGIGA)  THEN
         MKOUNT = MKOUNT + 1
         KOUNT = KOUNT - IGIGA
      ENDIF
      RETURN
C
C           ENTRY TO INPUT AND FLOAT INTEGER SEEDS FROM PREVIOUS RUN
      ENTRY RLUXIN(ISDEXT)
         TWOM24 = 1.
         DO 195 I= 1, 24
         NEXT(I) = I-1
  195    TWOM24 = TWOM24 * 0.5
         NEXT(1) = 24
         TWOM12 = TWOM24 * 4096.
      WRITE(6,'(A)') ' FULL INITIALIZATION OF RANLUX WITH 25 INTEGERS:'
      WRITE(6,'(5X,5I12)') ISDEXT
      DO 200 I= 1, 24
      SEEDS(I) = REAL(ISDEXT(I))*TWOM24
  200 CONTINUE
      CARRY = 0.
      IF (ISDEXT(25) .LT. 0)  CARRY = TWOM24
      ISD = IABS(ISDEXT(25))
      I24 = MOD(ISD,100)
      ISD = ISD/100
      J24 = MOD(ISD,100)
      ISD = ISD/100
      IN24 = MOD(ISD,100)
      ISD = ISD/100
      LUXLEV = ISD
        IF (LUXLEV .LE. MAXLEV) THEN
          NSKIP = NDSKIP(LUXLEV)
          WRITE (6,'(A,I2)') ' RANLUX LUXURY LEVEL SET BY RLUXIN TO: ',
     +                         LUXLEV
        ELSE  IF (LUXLEV .GE. 24) THEN
          NSKIP = LUXLEV - 24
          WRITE (6,'(A,I5)') ' RANLUX P-VALUE SET BY RLUXIN TO:',LUXLEV
        ELSE
          NSKIP = NDSKIP(MAXLEV)
          WRITE (6,'(A,I5)') ' RANLUX ILLEGAL LUXURY RLUXIN: ',LUXLEV
          LUXLEV = MAXLEV
        ENDIF
      INSEED = -1
      RETURN
C
C                    ENTRY TO OUPUT SEEDS AS INTEGERS
      ENTRY RLUXUT(ISDEXT)
      DO 300 I= 1, 24
         ISDEXT(I) = INT(SEEDS(I)*TWOP12*TWOP12)
  300 CONTINUE
      ISDEXT(25) = I24 + 100*J24 + 10000*IN24 + 1000000*LUXLEV
      IF (CARRY .GT. 0.)  ISDEXT(25) = -ISDEXT(25)
      RETURN
C
C                    ENTRY TO OUTPUT THE "CONVENIENT" RESTART POINT
      ENTRY RLUXAT(LOUT,INOUT,K1,K2)
      LOUT = LUXLEV
      INOUT = INSEED
      K1 = KOUNT
      K2 = MKOUNT
      RETURN
C
C                    ENTRY TO INITIALIZE FROM ONE OR THREE INTEGERS
      ENTRY RLUXGO(LUX,INS,K1,K2)
         IF (LUX .LT. 0) THEN
            LUXLEV = LXDFLT
         ELSE IF (LUX .LE. MAXLEV) THEN
            LUXLEV = LUX
         ELSE IF (LUX .LT. 24 .OR. LUX .GT. 2000) THEN
            LUXLEV = MAXLEV
            WRITE (6,'(A,I7)') ' RANLUX ILLEGAL LUXURY RLUXGO: ',LUX
         ELSE
            LUXLEV = LUX
            DO 310 ILX= 0, MAXLEV
              IF (LUX .EQ. NDSKIP(ILX)+24)  LUXLEV = ILX
  310       CONTINUE
         ENDIF
      IF (LUXLEV .LE. MAXLEV)  THEN
         NSKIP = NDSKIP(LUXLEV)
         WRITE(6,'(A,I2,A,I4)') ' RANLUX LUXURY LEVEL SET BY RLUXGO :',
     +        LUXLEV,'     P=', NSKIP+24
      ELSE
          NSKIP = LUXLEV - 24
          WRITE (6,'(A,I5)') ' RANLUX P-VALUE SET BY RLUXGO TO:',LUXLEV
      ENDIF
      IN24 = 0
      IF (INS .LT. 0)  WRITE (6,'(A)')
     +   ' ILLEGAL INITIALIZATION BY RLUXGO, NEGATIVE INPUT SEED'
      IF (INS .GT. 0)  THEN
       JSEED = INS
        WRITE(6,'(A,3I12)') ' RANLUX INITIALIZED BY RLUXGO FROM SEEDS',
     +      JSEED, K1,K2
      ELSE
        JSEED = JSDFLT
        WRITE(6,'(A)')' RANLUX INITIALIZED BY RLUXGO FROM DEFAULT SEED'
      ENDIF
      INSEED = JSEED
      NOTYET = .FALSE.
      TWOM24 = 1.
         DO 325 I= 1, 24
           TWOM24 = TWOM24 * 0.5
         K = JSEED/53668
         JSEED = 40014*(JSEED-K*53668) -K*12211
         IF (JSEED .LT. 0)  JSEED = JSEED+ICONS
         ISEEDS(I) = MOD(JSEED,ITWO24)
  325    CONTINUE
      TWOM12 = TWOM24 * 4096.
         DO 350 I= 1,24
         SEEDS(I) = REAL(ISEEDS(I))*TWOM24
         NEXT(I) = I-1
  350    CONTINUE
      NEXT(1) = 24
      I24 = 24
      J24 = 10
      CARRY = 0.
      IF (SEEDS(24) .EQ. 0.) CARRY = TWOM24
C        IF RESTARTING AT A BREAK POINT, SKIP K1 + IGIGA*K2
C        NOTE THAT THIS IS THE NUMBER OF NUMBERS DELIVERED TO
C        THE USER PLUS THE NUMBER SKIPPED (IF LUXURY .GT. 0).
      KOUNT = K1
      MKOUNT = K2
      IF (K1+K2 .NE. 0)  THEN
        DO 500 IOUTER= 1, K2+1
          INNER = IGIGA
          IF (IOUTER .EQ. K2+1)  INNER = K1
          DO 450 ISK= 1, INNER
            UNI = SEEDS(J24) - SEEDS(I24) - CARRY
            IF (UNI .LT. 0.)  THEN
               UNI = UNI + 1.0
               CARRY = TWOM24
            ELSE
               CARRY = 0.
            ENDIF
            SEEDS(I24) = UNI
            I24 = NEXT(I24)
            J24 = NEXT(J24)
  450     CONTINUE
  500   CONTINUE
C         GET THE RIGHT VALUE OF IN24 BY DIRECT CALCULATION
        IN24 = MOD(KOUNT, NSKIP+24)
        IF (MKOUNT .GT. 0)  THEN
           IZIP = MOD(IGIGA, NSKIP+24)
           IZIP2 = MKOUNT*IZIP + IN24
           IN24 = MOD(IZIP2, NSKIP+24)
        ENDIF
C       NOW IN24 HAD BETTER BE BETWEEN ZERO AND 23 INCLUSIVE
        IF (IN24 .GT. 23) THEN
           WRITE (6,'(A/A,3I11,A,I5)')
     +    '  ERROR IN RESTARTING WITH RLUXGO:','  THE VALUES', INS,
     +     K1, K2, ' CANNOT OCCUR AT LUXURY LEVEL', LUXLEV
           IN24 = 0
        ENDIF
      ENDIF
      RETURN
      END
C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C **********************************************************************
C
C **********************************************************************
